@@ -27,7 +27,6 @@
 #include <gst/rtp/gstrtpbuffer.h>
 
 #include <string.h>
-#include "gstisomp4elements.h"
 #include "gstrtpxqtdepay.h"
 
 #define MAKE_TLV(a,b)  (((a)<<8)|(b))
@@ -59,7 +58,6 @@
 #define QT_UINT64(a)  ((((guint64)QT_UINT32(a))<<32)|QT_UINT32(((guint8 *)a)+4))
 
 #define FOURCC_avc1     GST_MAKE_FOURCC('a','v','c','1')
-#define FOURCC_avc3     GST_MAKE_FOURCC('a','v','c','3')
 #define FOURCC_avcC     GST_MAKE_FOURCC('a','v','c','C')
 
 GST_DEBUG_CATEGORY_STATIC (rtpxqtdepay_debug);
@@ -74,7 +72,7 @@ enum
 
 enum
 {
-  PROP_0,
+  ARG_0,
 };
 
 static GstStaticPadTemplate gst_rtp_xqt_depay_src_template =
@@ -95,8 +93,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 #define gst_rtp_xqt_depay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpXQTDepay, gst_rtp_xqt_depay, GST_TYPE_RTP_BASE_DEPAYLOAD);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpxqtdepay, "rtpxqtdepay",
-    GST_RANK_MARGINAL, GST_TYPE_RTP_XQT_DEPAY, isomp4_element_init (plugin));
 
 static void gst_rtp_xqt_depay_finalize (GObject * object);
 
@@ -132,10 +128,10 @@ gst_rtp_xqt_depay_class_init (GstRtpXQTDepayClass * klass)
   GST_DEBUG_CATEGORY_INIT (rtpxqtdepay_debug, "rtpxqtdepay", 0,
       "QT Media RTP Depayloader");
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_xqt_depay_src_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_rtp_xqt_depay_sink_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_xqt_depay_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_rtp_xqt_depay_sink_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "RTP packet depayloader", "Codec/Depayloader/Network",
@@ -146,9 +142,6 @@ gst_rtp_xqt_depay_class_init (GstRtpXQTDepayClass * klass)
 static void
 gst_rtp_xqt_depay_init (GstRtpXQTDepay * rtpxqtdepay)
 {
-  gst_rtp_base_depayload_set_aggregate_hdrext_enabled (GST_RTP_BASE_DEPAYLOAD
-      (rtpxqtdepay), TRUE);
-
   rtpxqtdepay->adapter = gst_adapter_new ();
 }
 
@@ -186,7 +179,6 @@ gst_rtp_quicktime_parse_sd (GstRtpXQTDepay * rtpxqtdepay, guint8 * data,
 
   switch (fourcc) {
     case FOURCC_avc1:
-    case FOURCC_avc3:
     {
       guint32 chlen;
 
@@ -554,7 +546,6 @@ gst_rtp_xqt_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
       case 2:
       {
         guint slen;
-        GstBufferList *outbufs = NULL;
 
         /* multiple samples per packet. 
          *                      1                   2                   3
@@ -575,7 +566,6 @@ gst_rtp_xqt_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
          * . ......                                                        .
          * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
          */
-        outbufs = gst_buffer_list_new ();
         while (payload_len > 8) {
           s = (payload[0] & 0x80) != 0; /* contains sync sample */
           slen = (payload[2] << 8) | payload[3];
@@ -595,7 +585,7 @@ gst_rtp_xqt_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
           if (!s)
             GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DELTA_UNIT);
 
-          gst_buffer_list_add (outbufs, outbuf);
+          gst_rtp_base_depayload_push (depayload, outbuf);
 
           /* aligned on 32 bit boundary */
           slen = GST_ROUND_UP_4 (slen);
@@ -603,8 +593,6 @@ gst_rtp_xqt_depay_process (GstRTPBaseDepayload * depayload, GstBuffer * buf)
           payload += slen;
           payload_len -= slen;
         }
-        gst_rtp_base_depayload_push_list (depayload, outbufs);
-        outbuf = NULL;
         break;
       }
       case 3:
@@ -637,35 +625,30 @@ done:
 need_resync:
   {
     GST_DEBUG_OBJECT (rtpxqtdepay, "waiting for marker");
-    gst_rtp_base_depayload_dropped (depayload);
     goto done;
   }
 wrong_version:
   {
     GST_ELEMENT_WARNING (rtpxqtdepay, STREAM, DECODE,
         ("Unknown payload version."), (NULL));
-    gst_rtp_base_depayload_dropped (depayload);
     goto done;
   }
 pck_reserved:
   {
     GST_ELEMENT_WARNING (rtpxqtdepay, STREAM, DECODE,
         ("PCK reserved 0."), (NULL));
-    gst_rtp_base_depayload_dropped (depayload);
     goto done;
   }
 wrong_length:
   {
     GST_ELEMENT_WARNING (rtpxqtdepay, STREAM, DECODE,
         ("Wrong payload length."), (NULL));
-    gst_rtp_base_depayload_dropped (depayload);
     goto done;
   }
 unknown_format:
   {
     GST_ELEMENT_WARNING (rtpxqtdepay, STREAM, DECODE,
         ("Unknown payload format."), (NULL));
-    gst_rtp_base_depayload_dropped (depayload);
     goto done;
   }
 }
