@@ -24,10 +24,6 @@
 #include <unistd.h>             /* for getpid */
 #include "gstosxaudiosink.h"
 
-#if defined(MAC_OS_X_VERSION_MAX_ALLOWED) && MAC_OS_X_VERSION_MAX_ALLOWED < 120000
-#define kAudioObjectPropertyElementMain kAudioObjectPropertyElementMaster
-#endif
-
 static inline gboolean
 _audio_system_set_runloop (CFRunLoopRef runLoop)
 {
@@ -38,7 +34,7 @@ _audio_system_set_runloop (CFRunLoopRef runLoop)
   AudioObjectPropertyAddress runloopAddress = {
     kAudioHardwarePropertyRunLoop,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectSetPropertyData (kAudioObjectSystemObject,
@@ -46,114 +42,85 @@ _audio_system_set_runloop (CFRunLoopRef runLoop)
   if (status == noErr) {
     res = TRUE;
   } else {
-    GST_ERROR ("failed to set runloop to %p: %d", runLoop, (int) status);
+    GST_ERROR ("failed to set runloop to %p: %" GST_FOURCC_FORMAT,
+        runLoop, GST_FOURCC_ARGS (status));
   }
 
   return res;
 }
 
-typedef struct
+static inline AudioDeviceID
+_audio_system_get_default_output (void)
 {
-  AudioDeviceID id;
-  char *unique_id;
-} GstOsxAudioDevice;
-
-static void
-_free_gst_osx_audio_device (gpointer data)
-{
-  GstOsxAudioDevice *d = data;
-  g_free (d->unique_id);
-  g_free (d);
-}
-
-static inline GstOsxAudioDevice *
-_audio_system_get_default_device (gboolean output, const char *audio_type)
-{
-  GstOsxAudioDevice *ret = NULL;
+  OSStatus status = noErr;
+  UInt32 propertySize = sizeof (AudioDeviceID);
   AudioDeviceID device_id = kAudioDeviceUnknown;
 
-  device_id = gst_core_audio_device_get_prop_uint32 (kAudioObjectSystemObject,
-      output ? kAudioHardwarePropertyDefaultOutputDevice
-      : kAudioHardwarePropertyDefaultInputDevice);
-  if (device_id == kAudioDeviceUnknown) {
-    GST_ERROR ("Failed to get default %s device", audio_type);
-    return NULL;
+  AudioObjectPropertyAddress defaultDeviceAddress = {
+    kAudioHardwarePropertyDefaultOutputDevice,
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
+  };
+
+  status = AudioObjectGetPropertyData (kAudioObjectSystemObject,
+      &defaultDeviceAddress, 0, NULL, &propertySize, &device_id);
+  if (status != noErr) {
+    GST_ERROR ("failed getting default output device: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
   }
 
-  ret = g_new0 (GstOsxAudioDevice, 1);
-  ret->id = device_id;
-  ret->unique_id = gst_core_audio_device_get_prop_str (device_id,
-      kAudioDevicePropertyDeviceUID);
-
-  GST_DEBUG ("Default audio %s device: %d: %s", audio_type, ret->id,
-      ret->unique_id);
-
-  return ret;
+  return device_id;
 }
 
-static inline GPtrArray *
+static inline AudioDeviceID *
 _audio_system_get_devices (gint * ndevices)
 {
   OSStatus status = noErr;
   UInt32 propertySize = 0;
   AudioDeviceID *devices = NULL;
-  GPtrArray *ret = g_ptr_array_new_with_free_func (_free_gst_osx_audio_device);
 
   AudioObjectPropertyAddress audioDevicesAddress = {
     kAudioHardwarePropertyDevices,
-    kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyDataSize (kAudioObjectSystemObject,
       &audioDevicesAddress, 0, NULL, &propertySize);
   if (status != noErr) {
-    GST_WARNING ("failed getting number of devices: %d", (int) status);
+    GST_WARNING ("failed getting number of devices: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     return NULL;
   }
 
   *ndevices = propertySize / sizeof (AudioDeviceID);
 
   devices = (AudioDeviceID *) g_malloc (propertySize);
-
-  status = AudioObjectGetPropertyData (kAudioObjectSystemObject,
-      &audioDevicesAddress, 0, NULL, &propertySize, devices);
-  if (status != noErr) {
-    GST_WARNING ("failed getting the list of devices: %d", (int) status);
-    g_free (devices);
-    *ndevices = 0;
-    return ret;
+  if (devices) {
+    status = AudioObjectGetPropertyData (kAudioObjectSystemObject,
+        &audioDevicesAddress, 0, NULL, &propertySize, devices);
+    if (status != noErr) {
+      GST_WARNING ("failed getting the list of devices: %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
+      g_free (devices);
+      *ndevices = 0;
+      return NULL;
+    }
   }
-
-  for (int i = 0; i < *ndevices; i++) {
-    GstOsxAudioDevice *d = g_new0 (GstOsxAudioDevice, 1);
-    d->id = devices[i];
-    d->unique_id = gst_core_audio_device_get_prop_str (d->id,
-        kAudioDevicePropertyDeviceUID);
-    g_ptr_array_add (ret, d);
-    GST_DEBUG ("Found device '%s' id %i", d->unique_id, d->id);
-  }
-
-  g_free (devices);
-
-  return ret;
+  return devices;
 }
 
 static inline gboolean
-_audio_device_is_alive (AudioDeviceID device_id, gboolean output)
+_audio_device_is_alive (AudioDeviceID device_id)
 {
   OSStatus status = noErr;
   int alive = FALSE;
   UInt32 propertySize = sizeof (alive);
-  AudioObjectPropertyScope prop_scope;
-
-  prop_scope = output ? kAudioDevicePropertyScopeOutput :
-      kAudioDevicePropertyScopeInput;
 
   AudioObjectPropertyAddress audioDeviceAliveAddress = {
     kAudioDevicePropertyDeviceIsAlive,
-    prop_scope,
-    kAudioObjectPropertyElementMain
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyData (device_id,
@@ -165,25 +132,6 @@ _audio_device_is_alive (AudioDeviceID device_id, gboolean output)
   return alive;
 }
 
-static inline gboolean
-_audio_device_is_hidden (AudioDeviceID device_id)
-{
-  OSStatus status = noErr;
-  UInt32 hidden = FALSE;
-  UInt32 property_size = sizeof (hidden);
-  AudioObjectPropertyAddress property_address;
-
-  property_address.mSelector = kAudioDevicePropertyIsHidden;
-
-  status = AudioObjectGetPropertyData (device_id,
-      &property_address, 0, NULL, &property_size, &hidden);
-  if (status != noErr) {
-    return FALSE;
-  }
-
-  return hidden;
-}
-
 static inline guint
 _audio_device_get_latency (AudioDeviceID device_id)
 {
@@ -194,13 +142,14 @@ _audio_device_get_latency (AudioDeviceID device_id)
   AudioObjectPropertyAddress audioDeviceLatencyAddress = {
     kAudioDevicePropertyLatency,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyData (device_id,
       &audioDeviceLatencyAddress, 0, NULL, &propertySize, &latency);
   if (status != noErr) {
-    GST_ERROR ("failed to get latency: %d", (int) status);
+    GST_ERROR ("failed to get latency: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     latency = -1;
   }
 
@@ -217,13 +166,14 @@ _audio_device_get_hog (AudioDeviceID device_id)
   AudioObjectPropertyAddress audioDeviceHogModeAddress = {
     kAudioDevicePropertyHogMode,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyData (device_id,
       &audioDeviceHogModeAddress, 0, NULL, &propertySize, &hog_pid);
   if (status != noErr) {
-    GST_ERROR ("failed to get hog: %d", (int) status);
+    GST_ERROR ("failed to get hog: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     hog_pid = -1;
   }
 
@@ -240,7 +190,7 @@ _audio_device_set_hog (AudioDeviceID device_id, pid_t hog_pid)
   AudioObjectPropertyAddress audioDeviceHogModeAddress = {
     kAudioDevicePropertyHogMode,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectSetPropertyData (device_id,
@@ -249,7 +199,8 @@ _audio_device_set_hog (AudioDeviceID device_id, pid_t hog_pid)
   if (status == noErr) {
     res = TRUE;
   } else {
-    GST_ERROR ("failed to set hog: %d", (int) status);
+    GST_ERROR ("failed to set hog: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
   }
 
   return res;
@@ -266,7 +217,7 @@ _audio_device_set_mixing (AudioDeviceID device_id, gboolean enable_mix)
   AudioObjectPropertyAddress audioDeviceSupportsMixingAddress = {
     kAudioDevicePropertySupportsMixing,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   if (AudioObjectHasProperty (device_id, &audioDeviceSupportsMixingAddress)) {
@@ -274,17 +225,20 @@ _audio_device_set_mixing (AudioDeviceID device_id, gboolean enable_mix)
     status = AudioObjectIsPropertySettable (device_id,
         &audioDeviceSupportsMixingAddress, &writable);
     if (status) {
-      GST_DEBUG ("AudioObjectIsPropertySettable: %d", (int) status);
+      GST_DEBUG ("AudioObjectIsPropertySettable: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
     }
     status = AudioObjectGetPropertyDataSize (device_id,
         &audioDeviceSupportsMixingAddress, 0, NULL, &propertySize);
     if (status) {
-      GST_DEBUG ("AudioObjectGetPropertyDataSize: %d", (int) status);
+      GST_DEBUG ("AudioObjectGetPropertyDataSize: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
     }
     status = AudioObjectGetPropertyData (device_id,
         &audioDeviceSupportsMixingAddress, 0, NULL, &propertySize, &can_mix);
     if (status) {
-      GST_DEBUG ("AudioObjectGetPropertyData: %d", (int) status);
+      GST_DEBUG ("AudioObjectGetPropertyData: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
     }
 
     if (status == noErr && writable) {
@@ -295,15 +249,49 @@ _audio_device_set_mixing (AudioDeviceID device_id, gboolean enable_mix)
     }
 
     if (status != noErr) {
-      GST_ERROR ("failed to set mixmode: %d", (int) status);
+      GST_ERROR ("failed to set mixmode: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
     }
   } else {
-    GST_DEBUG ("property not found, mixing couldn't be changed");
+    GST_DEBUG ("property not found, mixing coudln't be changed");
   }
 
   return res;
 }
 
+static inline gchar *
+_audio_device_get_name (AudioDeviceID device_id)
+{
+  OSStatus status = noErr;
+  UInt32 propertySize = 0;
+  gchar *device_name = NULL;
+
+  AudioObjectPropertyAddress deviceNameAddress = {
+    kAudioDevicePropertyDeviceName,
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
+  };
+
+  /* Get the length of the device name */
+  status = AudioObjectGetPropertyDataSize (device_id,
+      &deviceNameAddress, 0, NULL, &propertySize);
+  if (status != noErr) {
+    goto beach;
+  }
+
+  /* Get the name of the device */
+  device_name = (gchar *) g_malloc (propertySize);
+  status = AudioObjectGetPropertyData (device_id,
+      &deviceNameAddress, 0, NULL, &propertySize, device_name);
+  if (status != noErr) {
+    g_free (device_name);
+    device_name = NULL;
+  }
+
+beach:
+  return device_name;
+}
+
 static inline gboolean
 _audio_device_has_output (AudioDeviceID device_id)
 {
@@ -313,7 +301,7 @@ _audio_device_has_output (AudioDeviceID device_id)
   AudioObjectPropertyAddress streamsAddress = {
     kAudioDevicePropertyStreams,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyDataSize (device_id,
@@ -328,6 +316,70 @@ _audio_device_has_output (AudioDeviceID device_id)
   return TRUE;
 }
 
+AudioChannelLayout *
+gst_core_audio_audio_device_get_channel_layout (AudioDeviceID device_id)
+{
+  OSStatus status = noErr;
+  UInt32 propertySize = 0;
+  AudioChannelLayout *layout = NULL;
+
+  AudioObjectPropertyAddress channelLayoutAddress = {
+    kAudioDevicePropertyPreferredChannelLayout,
+    kAudioDevicePropertyScopeOutput,
+    kAudioObjectPropertyElementMaster
+  };
+
+  /* Get the length of the default channel layout structure */
+  status = AudioObjectGetPropertyDataSize (device_id,
+      &channelLayoutAddress, 0, NULL, &propertySize);
+  if (status != noErr) {
+    GST_ERROR ("failed to get prefered layout: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
+    goto beach;
+  }
+
+  /* Get the default channel layout of the device */
+  layout = (AudioChannelLayout *) g_malloc (propertySize);
+  status = AudioObjectGetPropertyData (device_id,
+      &channelLayoutAddress, 0, NULL, &propertySize, layout);
+  if (status != noErr) {
+    GST_ERROR ("failed to get prefered layout: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
+    goto failed;
+  }
+
+  if (layout->mChannelLayoutTag == kAudioChannelLayoutTag_UseChannelBitmap) {
+    /* bitmap defined channellayout */
+    status =
+        AudioFormatGetProperty (kAudioFormatProperty_ChannelLayoutForBitmap,
+        sizeof (UInt32), &layout->mChannelBitmap, &propertySize, layout);
+    if (status != noErr) {
+      GST_ERROR ("failed to get layout for bitmap: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
+      goto failed;
+    }
+  } else if (layout->mChannelLayoutTag !=
+      kAudioChannelLayoutTag_UseChannelDescriptions) {
+    /* layouttags defined channellayout */
+    status = AudioFormatGetProperty (kAudioFormatProperty_ChannelLayoutForTag,
+        sizeof (AudioChannelLayoutTag), &layout->mChannelLayoutTag,
+        &propertySize, layout);
+    if (status != noErr) {
+      GST_ERROR ("failed to get layout for tag: %" GST_FOURCC_FORMAT,
+          GST_FOURCC_ARGS (status));
+      goto failed;
+    }
+  }
+
+beach:
+  gst_core_audio_dump_channel_layout (layout);
+  return layout;
+
+failed:
+  g_free (layout);
+  return NULL;
+}
+
 static inline AudioStreamID *
 _audio_device_get_streams (AudioDeviceID device_id, gint * nstreams)
 {
@@ -338,13 +390,14 @@ _audio_device_get_streams (AudioDeviceID device_id, gint * nstreams)
   AudioObjectPropertyAddress streamsAddress = {
     kAudioDevicePropertyStreams,
     kAudioDevicePropertyScopeOutput,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyDataSize (device_id,
       &streamsAddress, 0, NULL, &propertySize);
   if (status != noErr) {
-    GST_WARNING ("failed getting number of streams: %d", (int) status);
+    GST_WARNING ("failed getting number of streams: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     return NULL;
   }
 
@@ -355,7 +408,8 @@ _audio_device_get_streams (AudioDeviceID device_id, gint * nstreams)
     status = AudioObjectGetPropertyData (device_id,
         &streamsAddress, 0, NULL, &propertySize, streams);
     if (status != noErr) {
-      GST_WARNING ("failed getting the list of streams: %d", (int) status);
+      GST_WARNING ("failed getting the list of streams: %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
       g_free (streams);
       *nstreams = 0;
       return NULL;
@@ -375,13 +429,14 @@ _audio_stream_get_latency (AudioStreamID stream_id)
   AudioObjectPropertyAddress latencyAddress = {
     kAudioStreamPropertyLatency,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyData (stream_id,
       &latencyAddress, 0, NULL, &propertySize, &latency);
   if (status != noErr) {
-    GST_ERROR ("failed to get latency: %d", (int) status);
+    GST_ERROR ("failed to get latency: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     latency = -1;
   }
 
@@ -398,13 +453,14 @@ _audio_stream_get_current_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyData (stream_id,
       &formatAddress, 0, NULL, &propertySize, format);
   if (status != noErr) {
-    GST_ERROR ("failed to get current format: %d", (int) status);
+    GST_ERROR ("failed to get current format: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     return FALSE;
   }
 
@@ -421,13 +477,14 @@ _audio_stream_set_current_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectSetPropertyData (stream_id,
       &formatAddress, 0, NULL, propertySize, &format);
   if (status != noErr) {
-    GST_ERROR ("failed to set current format: %d", (int) status);
+    GST_ERROR ("failed to set current format: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     return FALSE;
   }
 
@@ -444,13 +501,14 @@ _audio_stream_get_formats (AudioStreamID stream_id, gint * nformats)
   AudioObjectPropertyAddress formatsAddress = {
     kAudioStreamPropertyAvailablePhysicalFormats,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   status = AudioObjectGetPropertyDataSize (stream_id,
       &formatsAddress, 0, NULL, &propertySize);
   if (status != noErr) {
-    GST_WARNING ("failed getting number of stream formats: %d", (int) status);
+    GST_WARNING ("failed getting number of stream formats: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     return NULL;
   }
 
@@ -461,8 +519,8 @@ _audio_stream_get_formats (AudioStreamID stream_id, gint * nformats)
     status = AudioObjectGetPropertyData (stream_id,
         &formatsAddress, 0, NULL, &propertySize, formats);
     if (status != noErr) {
-      GST_WARNING ("failed getting the list of stream formats: %d",
-          (int) status);
+      GST_WARNING ("failed getting the list of stream formats: %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
       g_free (formats);
       *nformats = 0;
       return NULL;
@@ -531,7 +589,7 @@ _audio_stream_change_format (AudioStreamID stream_id,
   AudioObjectPropertyAddress formatAddress = {
     kAudioStreamPropertyPhysicalFormat,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   GST_DEBUG ("setting stream format: " CORE_AUDIO_FORMAT,
@@ -547,7 +605,8 @@ _audio_stream_change_format (AudioStreamID stream_id,
   status = AudioObjectAddPropertyListener (stream_id, &formatAddress,
       _audio_stream_format_listener, (void *) &prop_mutex);
   if (status != noErr) {
-    GST_ERROR ("AudioObjectAddPropertyListener failed: %d", (int) status);
+    GST_ERROR ("AudioObjectAddPropertyListener failed: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     goto done;
   }
 
@@ -560,11 +619,12 @@ _audio_stream_change_format (AudioStreamID stream_id,
    * it is also not atomic in its behaviour.
    * Therefore we check 4 times before we really give up. */
   for (i = 0; i < 4; i++) {
-    gint64 timeout;
+    GTimeVal timeout;
 
-    timeout = g_get_monotonic_time () + 250000;
+    g_get_current_time (&timeout);
+    g_time_val_add (&timeout, 250000);
 
-    if (!g_cond_wait_until (&prop_mutex.cond, &prop_mutex.lock, timeout)) {
+    if (!g_cond_wait_until (&prop_mutex.cond, &prop_mutex.lock, timeout.tv_sec)) {
       GST_LOG ("timeout...");
     }
 
@@ -594,7 +654,8 @@ done:
   status = AudioObjectRemovePropertyListener (stream_id,
       &formatAddress, _audio_stream_format_listener, (void *) &prop_mutex);
   if (status != noErr) {
-    GST_ERROR ("AudioObjectRemovePropertyListener failed: %d", (int) status);
+    GST_ERROR ("AudioObjectRemovePropertyListener failed: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
   }
   /* Destroy the lock and condition */
   g_mutex_unlock (&prop_mutex.lock);
@@ -637,7 +698,7 @@ _monitorize_spdif (GstCoreAudio * core_audio)
   AudioObjectPropertyAddress propAddress = {
     kAudioDevicePropertyDeviceHasChanged,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   /* Install the property listener */
@@ -646,7 +707,8 @@ _monitorize_spdif (GstCoreAudio * core_audio)
       (void *) core_audio);
   if (status != noErr) {
     GST_ERROR_OBJECT (core_audio->osxbuf,
-        "AudioObjectAddPropertyListener failed: %d", (int) status);
+        "AudioObjectAddPropertyListener failed: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     ret = FALSE;
   }
 
@@ -662,7 +724,7 @@ _unmonitorize_spdif (GstCoreAudio * core_audio)
   AudioObjectPropertyAddress propAddress = {
     kAudioDevicePropertyDeviceHasChanged,
     kAudioObjectPropertyScopeGlobal,
-    kAudioObjectPropertyElementMain
+    kAudioObjectPropertyElementMaster
   };
 
   /* Remove the property listener */
@@ -671,7 +733,8 @@ _unmonitorize_spdif (GstCoreAudio * core_audio)
       (void *) core_audio);
   if (status != noErr) {
     GST_ERROR_OBJECT (core_audio->osxbuf,
-        "AudioObjectRemovePropertyListener failed: %d", (int) status);
+        "AudioObjectRemovePropertyListener failed: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
     ret = FALSE;
   }
 
@@ -869,7 +932,8 @@ _remove_render_spdif_callback (GstCoreAudio * core_audio)
       AudioDeviceDestroyIOProcID (core_audio->device_id, core_audio->procID);
   if (status != noErr) {
     GST_ERROR_OBJECT (core_audio->osxbuf,
-        "AudioDeviceDestroyIOProcID failed: %d", (int) status);
+        "AudioDeviceDestroyIOProcID failed: %"
+        GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
   }
 
   GST_DEBUG_OBJECT (core_audio,
@@ -878,7 +942,7 @@ _remove_render_spdif_callback (GstCoreAudio * core_audio)
 
   /* We're deactivated.. */
   core_audio->procID = 0;
-  g_atomic_int_set (&core_audio->io_proc_dropping, FALSE);
+  core_audio->io_proc_needs_deactivation = FALSE;
   core_audio->io_proc_active = FALSE;
 }
 
@@ -898,19 +962,21 @@ _io_proc_spdif_start (GstCoreAudio * core_audio)
         (void *) core_audio, &core_audio->procID);
     if (status != noErr) {
       GST_ERROR_OBJECT (core_audio->osxbuf,
-          ":AudioDeviceCreateIOProcID failed: %d", (int) status);
+          ":AudioDeviceCreateIOProcID failed: %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
       return FALSE;
     }
     core_audio->io_proc_active = TRUE;
   }
 
-  g_atomic_int_set (&core_audio->io_proc_dropping, FALSE);
+  core_audio->io_proc_needs_deactivation = FALSE;
 
   /* Start device */
   status = AudioDeviceStart (core_audio->device_id, core_audio->procID);
   if (status != noErr) {
     GST_ERROR_OBJECT (core_audio->osxbuf,
-        "AudioDeviceStart failed: %d", (int) status);
+        "AudioDeviceStart failed: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
     return FALSE;
   }
   return TRUE;
@@ -925,7 +991,8 @@ _io_proc_spdif_stop (GstCoreAudio * core_audio)
   status = AudioDeviceStop (core_audio->device_id, core_audio->procID);
   if (status != noErr) {
     GST_ERROR_OBJECT (core_audio->osxbuf,
-        "AudioDeviceStop failed: %d", (int) status);
+        "AudioDeviceStop failed: %" GST_FOURCC_FORMAT,
+        GST_FOURCC_ARGS (status));
   }
 
   GST_DEBUG_OBJECT (core_audio,
@@ -949,8 +1016,6 @@ _io_proc_spdif_stop (GstCoreAudio * core_audio)
 static gboolean
 gst_core_audio_open_impl (GstCoreAudio * core_audio)
 {
-  gboolean ret;
-
   /* The following is needed to instruct HAL to create their own
    * thread to handle the notifications. */
   _audio_system_set_runloop (NULL);
@@ -965,21 +1030,8 @@ gst_core_audio_open_impl (GstCoreAudio * core_audio)
    * we will do input with it.
    * http://developer.apple.com/technotes/tn2002/tn2091.html
    */
-  ret = gst_core_audio_open_device (core_audio, kAudioUnitSubType_HALOutput,
+  return gst_core_audio_open_device (core_audio, kAudioUnitSubType_HALOutput,
       "HALOutput");
-  if (!ret) {
-    GST_DEBUG ("Could not open device");
-    goto done;
-  }
-
-  ret = gst_core_audio_bind_device (core_audio);
-  if (!ret) {
-    GST_DEBUG ("Could not bind device");
-    goto done;
-  }
-
-done:
-  return ret;
 }
 
 static gboolean
@@ -1008,13 +1060,11 @@ gst_core_audio_pause_processing_impl (GstCoreAudio * core_audio)
         "osx ring buffer pause ioproc: %p device_id %lu",
         core_audio->element->io_proc, (gulong) core_audio->device_id);
     if (core_audio->io_proc_active) {
-      /* CoreAudio isn't threadsafe enough to remove the render callback in the
-       * render notify callback, so we pause by having the callback drop
-       * samples or feed silence. See:
+      /* CoreAudio isn't threadsafe enough to do this here;
+       * we must deactivate the render callback elsewhere. See:
        * http://lists.apple.com/archives/Coreaudio-api/2006/Mar/msg00010.html
-       * https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/4155
        */
-      g_atomic_int_set (&core_audio->io_proc_dropping, TRUE);
+      core_audio->io_proc_needs_deactivation = TRUE;
     }
   }
   return TRUE;
@@ -1048,8 +1098,8 @@ gst_core_audio_get_samples_and_latency_impl (GstCoreAudio * core_audio,
         latency, &size);
 
     if (status) {
-      GST_WARNING_OBJECT (core_audio->osxbuf, "Failed to get latency: %d",
-          (int) status);
+      GST_WARNING_OBJECT (core_audio->osxbuf, "Failed to get latency: %"
+          GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
       *samples = 0;
       return FALSE;
     }
@@ -1065,15 +1115,6 @@ gst_core_audio_initialize_impl (GstCoreAudio * core_audio,
     gboolean is_passthrough, guint32 * frame_size)
 {
   gboolean ret = FALSE;
-  OSStatus status;
-
-  /* Uninitialize the AudioUnit before changing formats */
-  status = AudioUnitUninitialize (core_audio->audiounit);
-  if (status) {
-    GST_ERROR_OBJECT (core_audio, "Failed to uninitialize AudioUnit: %d",
-        (int) status);
-    return FALSE;
-  }
 
   core_audio->is_passthrough = is_passthrough;
   if (is_passthrough) {
@@ -1082,167 +1123,108 @@ gst_core_audio_initialize_impl (GstCoreAudio * core_audio,
     _monitorize_spdif (core_audio);
   } else {
     OSStatus status;
-    UInt32 propertySize = sizeof (*frame_size);
+    UInt32 propertySize;
 
     core_audio->stream_idx = 0;
     if (!gst_core_audio_set_format (core_audio, format))
       goto done;
 
-    if (!gst_core_audio_set_channel_layout (core_audio,
+    if (!gst_core_audio_set_channels_layout (core_audio,
             format.mChannelsPerFrame, caps))
       goto done;
 
-    // Attempt to configure the requested frame size if smaller than the device's
-    // This will apparently modify the size for all audio devices in the current process so should
-    // be done conservatively
-    if (frame_size != 0) {
-      guint32 cur_frame_size;
-      status = AudioUnitGetProperty (core_audio->audiounit, kAudioDevicePropertyBufferFrameSize, kAudioUnitScope_Global, 0,     /* N/A for global */
-          &cur_frame_size, &propertySize);
-      if (!status && *frame_size < cur_frame_size) {
-        status = AudioUnitSetProperty (core_audio->audiounit, kAudioDevicePropertyBufferFrameSize, kAudioUnitScope_Global, 0,   /* N/A for global */
-            frame_size, propertySize);
-        if (status) {
-          GST_WARNING_OBJECT (core_audio->osxbuf,
-              "Failed to set desired frame size of %u: %d", *frame_size,
-              (int) status);
-        }
-      }
-    }
-    status = AudioUnitGetProperty (core_audio->audiounit, kAudioDevicePropertyBufferFrameSize, kAudioUnitScope_Global, 0,       /* N/A for global */
-        frame_size, &propertySize);
-
-    if (status) {
-      GST_WARNING_OBJECT (core_audio->osxbuf, "Failed to get frame size: %d",
-          (int) status);
+    if (!gst_core_audio_bind_device (core_audio))
       goto done;
+
+    if (core_audio->is_src) {
+      propertySize = sizeof (*frame_size);
+      status = AudioUnitGetProperty (core_audio->audiounit, kAudioDevicePropertyBufferFrameSize, kAudioUnitScope_Global, 0,     /* N/A for global */
+          frame_size, &propertySize);
+
+      if (status) {
+        GST_WARNING_OBJECT (core_audio->osxbuf, "Failed to get frame size: %"
+            GST_FOURCC_FORMAT, GST_FOURCC_ARGS (status));
+        goto done;
+      }
     }
   }
 
   ret = TRUE;
 
 done:
-  /* Format changed, initialise the AudioUnit again */
-  status = AudioUnitInitialize (core_audio->audiounit);
-  if (status) {
-    GST_ERROR_OBJECT (core_audio, "Failed to initialize AudioUnit: %d",
-        (int) status);
-    ret = FALSE;
-  }
-
   if (ret) {
     GST_DEBUG_OBJECT (core_audio, "osxbuf ring buffer acquired");
   }
-
   return ret;
 }
 
 static gboolean
-gst_core_audio_select_device_impl (GstCoreAudio * core_audio)
+gst_core_audio_select_device_impl (AudioDeviceID * device_id)
 {
-  AudioDeviceID device_id = core_audio->device_id;
-  char *unique_id = core_audio->unique_id;
-  gboolean output = !core_audio->is_src;
-  const char *audio_type = output ? "output" : "input";
-  int i, ndevices = 0;
-  GPtrArray *devices = _audio_system_get_devices (&ndevices);
-  GstOsxAudioDevice *default_device =
-      _audio_system_get_default_device (output, audio_type);
+  AudioDeviceID *devices = NULL;
+  AudioDeviceID default_device_id = 0;
+  AudioChannelLayout *channel_layout;
+  gint i, ndevices = 0;
   gboolean res = FALSE;
 
+  devices = _audio_system_get_devices (&ndevices);
+
   if (ndevices < 1) {
-    GST_ERROR ("No audio devices found");
-    goto done;
-  }
-  GST_DEBUG ("Found %d audio device(s)", ndevices);
-
-  /* Prefer unique-id since that is more likely to be correct */
-  if (unique_id) {
-    device_id = kAudioDeviceUnknown;
-    for (i = 0; i < ndevices; i++) {
-      GstOsxAudioDevice *d = g_ptr_array_index (devices, i);
-      if (g_strcmp0 (unique_id, d->unique_id) == 0) {
-        device_id = d->id;
-        res = TRUE;
-        break;
-      }
-    }
-    if (res == FALSE) {
-      GST_ERROR ("No audio %s device with unique-id %s", audio_type, unique_id);
-    }
-    /* Skip the checks below for device_id being valid and just return */
+    GST_ERROR ("no audio output devices found");
     goto done;
   }
 
-  /* Here we decide if selected device_id is valid or autoselect
-   * the default one when required */
-  if (device_id == kAudioDeviceUnknown) {
-    if (default_device == NULL) {
-      GST_ERROR ("Cannot auto-select %s device, no default", audio_type);
-      res = FALSE;
-    } else if (default_device->id != kAudioDeviceUnknown) {
-      device_id = default_device->id;
-      unique_id = default_device->unique_id;
-      default_device->unique_id = NULL;
-      res = TRUE;
-    } else {
-      GST_ERROR ("No device of required type available");
-      res = FALSE;
-    }
-  } else if (_audio_device_is_hidden (device_id)) {
-    if (_audio_device_is_alive (device_id, output)) {
-      for (i = 0; i < ndevices; i++) {
-        GstOsxAudioDevice *d = g_ptr_array_index (devices, i);
-        if (device_id == d->id) {
-          unique_id = d->unique_id;
-          d->unique_id = NULL;
-          res = TRUE;
-          break;
+  GST_DEBUG ("found %d audio device(s)", ndevices);
+
+  for (i = 0; i < ndevices; i++) {
+    gchar *device_name;
+
+    if ((device_name = _audio_device_get_name (devices[i]))) {
+      if (!_audio_device_has_output (devices[i])) {
+        GST_DEBUG ("Input Device ID: %u Name: %s",
+            (unsigned) devices[i], device_name);
+      } else {
+        GST_DEBUG ("Output Device ID: %u Name: %s",
+            (unsigned) devices[i], device_name);
+
+        channel_layout =
+            gst_core_audio_audio_device_get_channel_layout (devices[i]);
+        if (channel_layout) {
+          gst_core_audio_dump_channel_layout (channel_layout);
+          g_free (channel_layout);
         }
       }
 
-      if (res == FALSE) {
-        GST_ERROR ("Requested hidden device with id %u not found", device_id);
-      }
-    } else {
-      GST_ERROR ("Requested hidden device not usable");
-      res = FALSE;
+      g_free (device_name);
+    }
+  }
+
+  /* Find the ID of the default output device */
+  default_device_id = _audio_system_get_default_output ();
+
+  /* Here we decide if selected device is valid or autoselect
+   * the default one when required */
+  if (*device_id == kAudioDeviceUnknown) {
+    if (default_device_id != kAudioDeviceUnknown) {
+      *device_id = default_device_id;
+      res = TRUE;
     }
   } else {
     for (i = 0; i < ndevices; i++) {
-      GstOsxAudioDevice *d = g_ptr_array_index (devices, i);
-      if (device_id == d->id) {
-        unique_id = d->unique_id;
-        d->unique_id = NULL;
+      if (*device_id == devices[i]) {
         res = TRUE;
-        break;
       }
     }
 
-    if (res && !_audio_device_is_alive (device_id, output)) {
+    if (res && !_audio_device_is_alive (*device_id)) {
       GST_ERROR ("Requested device not usable");
       res = FALSE;
+      goto done;
     }
   }
 
 done:
-  if (res) {
-    g_assert_cmpint (device_id, !=, kAudioDeviceUnknown);
-    g_assert (unique_id != NULL);
-    core_audio->device_id = device_id;
-    core_audio->is_default = (default_device && (device_id ==
-            default_device->id));
-    if (unique_id != core_audio->unique_id) {
-      g_free (core_audio->unique_id);
-      core_audio->unique_id = unique_id;
-    }
-  }
-
-  g_ptr_array_unref (devices);
-  if (default_device)
-    g_free (default_device->unique_id);
-  g_free (default_device);
-
+  g_free (devices);
   return res;
 }
 
@@ -1267,3 +1249,34 @@ gst_core_audio_audio_device_is_spdif_avail_impl (AudioDeviceID device_id)
 
   return res;
 }
+
+static gboolean
+gst_core_audio_select_source_device_impl (AudioDeviceID * device_id)
+{
+  OSStatus status;
+  UInt32 propertySize;
+
+  if (*device_id == kAudioDeviceUnknown) {
+    /* If no specific device has been selected by the user, then pick the
+     * default device */
+    GST_DEBUG ("Selecting device for OSXAudioSrc");
+    propertySize = sizeof (*device_id);
+    status = AudioHardwareGetProperty (kAudioHardwarePropertyDefaultInputDevice,
+        &propertySize, device_id);
+
+    if (status) {
+      GST_WARNING ("AudioHardwareGetProperty returned %d", (int) status);
+    } else {
+      GST_DEBUG ("AudioHardwareGetProperty returned 0");
+    }
+
+    if (*device_id == kAudioDeviceUnknown) {
+      GST_WARNING ("AudioHardwareGetProperty: device_id is "
+          "kAudioDeviceUnknown");
+    }
+
+    GST_DEBUG ("AudioHardwareGetProperty: device_id is %lu", (long) *device_id);
+  }
+
+  return TRUE;
+}
